(*
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *)

open Core
open Ast
open Expression
open Pyre

let is_local identifier = String.is_prefix ~prefix:"$" identifier

let extract_constant_name { Node.value = expression; _ } =
  match expression with
  | Expression.String literal -> Some literal.value
  | Integer i -> Some (string_of_int i)
  | Name name -> (
      let name = name_to_reference name >>| Reference.delocalize >>| Reference.last in
      match name with
      (* Heuristic: All uppercase names tend to be enums, so only taint the field in those cases. *)
      | Some name
        when String.for_all name ~f:(fun character ->
                 (not (Char.is_alpha character)) || Char.is_uppercase character) ->
          Some name
      | _ -> None )
  | _ -> None
